#########################################################################################
##  R code for "estimation of parametric failure time distributions based on 
##  interval-censored data with irregular dependent follow-up"
##  
##  Yayuan Zhu*, Jerald F. Lawless, and Cecilia A. Cotton  
##  Note: this code implements the IIV-weighted estimation and doubly weighted estimation
##  methods for model (5.1).
##  -------------------------------------------------------------------------------------
##  Platform, Bugs and Side-Effects:
##     -- R version 3.1.2 (2014-10-31) under Windows 64-bit
##     -- requires the following R packages:  nleqslv
##  -------------------------------------------------------------------------------------
##  Date: 11/5/2016
##  Corresponding Author: Yayuan Zhu, Ph.D.
##  Department of Statistics and Actuarial Science
##  University of Waterloo
##  Waterloo, ON N2L 3G1, Canada.
##  E-mail: yayuan.zhu@uwaterloo.ca
#########################################################################################

library(nleqslv)

surv.fn <- function(dataset, theta, grid){
    theta0 <- theta[1]
    theta1 <- theta[2]
    surv <- (1-exp(theta0+theta1*dataset$A))^(grid*dataset$visit.time)
    return(surv)
}


iivwGEE <- function(dataset, surv.fn, theta.init, grid){
    id.v <- sort(unique(dataset$id))
    gee.iiw <- function(theta){
       theta0 <- theta[1]
       theta1 <- theta[2]
       Ugee.fn <- 0
       for (i in 1:length(id.v)){
           sub.data <- subset(simu.data, id==id.v[i])
           sub.data <- sub.data[order(sub.data$visit.time),]
           S <- surv.fn(sub.data, theta, grid)
           g1 <- -grid*sub.data$visit.time*exp(theta0+theta1*sub.data$A)/((1-exp(theta0+theta1*sub.data$A))*(1-S))
           g2 <- -grid*sub.data$visit.time*exp(theta0+theta1*sub.data$A)*sub.data$A/((1-exp(theta0+theta1*sub.data$A))*(1-S))
           g  <- rbind(g1,g2)
           Ugee.plus <- g%*%((sub.data$Y-S)*sub.data$weight)
           Ugee.fn <- Ugee.fn + Ugee.plus
           rm(S,g1,g2,g,Ugee.plus)
       }
       return(Ugee.fn)
    }

    result.est <- nleqslv(theta.init, gee.iiw, method = "Newton", control=list(ftol=1e-8,maxit=100,xtol=1e-8)) 
    theta.est <- as.vector(result.est$x)
    return(theta.est)
}
 
 
theta.est <- iivwGEE(simu.data, surv.fn=surv.fn, theta.init=c(-1,-1), grid=100)
print(theta.est)
 
